(***************************************************)
(* Office ToolBar Components                       *)
(* Version 1.0                                     *)
(* ToTask Unit                                     *)
(***************************************************)
(*                                                 *)
(* (c)	Eric Lawrence                              *)
(*	Lead Programmer                            *)
(*	Delta Programming Group                    *)
(*                                                 *)
(*	deltagrp@juno.com                          *)
(*      deltagrp@keynetcorp.net                    *)
(*                                                 *)
(*                                                 *)
(* (c) 1998-99 Evgesoft                            *)
(* All right reserved.                             *)
(*                                                 *)
(***************************************************)
(* Internet: dev@evgesoft.com                      *)
(* WWW:                                            *)
(*   http://www.evgesoft.com                       *)
(***************************************************)
unit op_NoTask;

interface

uses
  Windows, Classes, Forms, SysUtils, Controls, Messages;

type

  TDuplicateComponent = class(Exception);
  { Define a Form not Owner object exception }
  TFormNotOwner = class(Exception);
  { The NoTask Class declaration }
  TopNoTask = class(TComponent)
  private
    { Private declarations - Attributes }
    FHideForm: boolean;
    OldWndProc: TFarProc;
    NewWndProc: Pointer;


    { Private declarations - Methods }

    Function IsIt:boolean;
    Procedure SetIt (value:boolean);

    {Start of Block to trap Parent form's messages...}
    procedure HookParent;
    procedure UnhookParent;
    procedure HookWndProc(var Message: TMessage);
    {End of Block to trap Parent form's messages...}

  protected

  public
    { Public declarations - Methods }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;override;
    procedure Loaded; override;
    procedure ProcessEnabled;
  published
    { Published declarations }
    property HideForm: boolean read IsIt write SetIt stored true default true;
  end;

implementation

destructor TopNoTask.Destroy;
begin
  { Always make sure that the hook is removed. }
  UnhookParent;
  inherited destroy;
end;


(*
**  Overridden constructor for our component.  Creates underlying TComponent,
**  sets our own method pointer to an unassigned state, and checks that there
**  is only one TopNoTask Component on the form.  If there is another one,
**  this one is destroyed by virtue of the exception handler within the design
**  mode.
*)

constructor TopNoTask.Create(AOwner: TComponent);
var
  i: word;				{ General Loop Counter }
  CompCount: byte;			{ TopNoTask Component count }
begin
  inherited Create(AOwner);             { Call TComponent Constructor }
  fhideform:=true;   //Default to hidden
  NewWndProc := nil;
  OldWndProc := nil;
  CompCount := 0;                       { Initialise Component Count to zero }
  { If we are designing at present }
  if (csDesigning in ComponentState) then
    if (AOwner is TForm) then
      with (AOwner as TForm) do
      begin
        for i := 0 to ComponentCount - 1 do { Check if there is already one of us! }
          if Components[i] is TopNoTask then inc(CompCount);

if CompCount > 1 then raise TDuplicateComponent.Create ('There is already a TopNoTask component on this Form');
     end
      else
        raise TFormNotOwner.Create
              ('The owner of TopNoTask Component is not a TForm');
        HookParent;

end;


(*
**  Overridden LOADED method.  This ensures that the check for a previous
**  instance of our Application is done immediately after the TopNoTask
**  components properties have been read in from the form, and initialized.
*)

procedure TopNoTask.Loaded;
begin
  inherited Loaded;                     { Always call inherited Loaded method }
 if not (csDesigning in ComponentState) then
  ProcessEnabled;
 end;


Procedure TopNoTask.ProcessEnabled;
begin
//if (owner as tform).windowstate<>wsMinimized then exit;
if fHideform then
showwindow(FindWindow (nil,@Application.Title[1]),sw_hide)
else
showwindow(FindWindow (nil,@Application.Title[1]),sw_restore);
end;


Function TopNoTask.IsIt:boolean;
begin
result:=fHideform;
end;

Procedure TopNoTask.SetIt(value:boolean);
begin
fhideform:=value;
processenabled;
end;

procedure TopNoTask.HookParent;
{ This procedure is used to get the parent's window procedure, save it,      }
{ and replace it with our own.  This allows see all of the parent's messages }
{ before it does.                                                            }
begin
  { If there is no parent, we can't hook it. }
 if owner=nil then exit;
  { Get the old window procedure via API call and store it. }
  OldWndProc := TFarProc(GetWindowLong((owner as tform).Handle, GWL_WNDPROC));
  { Convert our object method into something Windows knows how to call }
  NewWndProc := MakeObjectInstance(HookWndProc);
  { Install it as the new Parent window procedure }
  SetWindowLong((owner as tform).Handle, GWL_WNDPROC, LongInt(NewWndProc));
end;

procedure TopNoTask.UnhookParent;
{ Remove our window function and reinstall the original. }
begin
  { We must have a parent, and we must have already hooked it. }
  if (owner <> NIL) and assigned(OldWndProc) then
    { Set back to original window procedure }
    SetWindowLong((owner as tform).Handle, GWL_WNDPROC, LongInt(OldWndProc));
  { If we have created a window procedure via MakeObjectInstance, }
  { it must be disposed of.                                       }
  if assigned(NewWndProc) then
    FreeObjectInstance(NewWndProc);
  { Reset variables to NIL }
  NewWndProc := NIL;
  OldWndProc := NIL;
end;

{ The window procedure that is installed into our parent. }
procedure TopNoTask.HookWndProc(var Message: TMessage);

begin
  { If there's no parent, something has really gone wrong. }
  if owner = NIL then exit;
        if (message.msg=wm_showwindow) then
            begin
              if (message.wparam<>0) then {restoring...}
                   ProcessEnabled;
            end;

{ ALWAYS call the old window procedure so the parent can process its stuff  }
message.Result := CallWindowProc(OldWndProc, (owner as tform).Handle, message.Msg, message.wParam, message.lParam);

end;

end.

