(***************************************************)
(* Office ToolBar Components                       *)
(* Sounde Support                                  *)
(* Version 1.0                                     *)
(***************************************************)
(*                                                 *)
(* (c) 1998-99 Evgesoft                            *)
(* All right reserved.                             *)
(*                                                 *)
(***************************************************)
(* Internet: dev@evgesoft.com                      *)
(* WWW:                                            *)
(*   http://www.evgesoft.com                       *)
(***************************************************)
unit op_wave;

interface

uses
  Classes, SysUtils, Windows;

type

// WAV stream (stored in form)

  TWave = class(TPersistent)
  private
    FModified: Boolean;
    FWaveData: TMemoryStream;
    FOnChange: TNotifyEvent;
    function GetEmpty: Boolean;
    procedure SetModified(Value: Boolean);
  protected
    procedure ReadData(Stream: TStream); virtual;
    procedure WriteData(Stream: TStream); virtual;
    procedure Changed(Sender: TObject);
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure LoadFromFile(const Filename: string); virtual;
    procedure SaveToFile(const Filename: string); virtual;
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
    property Empty: Boolean read GetEmpty;
    property WaveData: TMemoryStream read FWaveData;
    property Modified: Boolean read FModified write SetModified;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

implementation

{ TWave }

{ class instance constructor }
constructor TWave.Create;
begin
  inherited Create;
  { create WaveAudio data buffer }
  FWaveData := TMemoryStream.Create;
end;

{ destroy class instance }
destructor TWave.Destroy;
begin
  FWaveData.Free;
  inherited Destroy;
end;

{ returns false if buffer is nil }
function TWave.GetEmpty;
begin
  Result := FWaveData = nil;
end;

{ method to copy WavwAudio data when required }
procedure TWave.Assign(Source: TPersistent);
begin
  if (Source = nil) or (Source is TWave) then
  begin
    if Source <> nil then
      FWaveData := TWave(Source).FWaveData;
    Changed(Self);
    Exit;
  end;
  inherited Assign(Source);
end;

{ method to load WaveAudio data from a stream - used
  by the library and by the LoadFromFile method }
procedure TWave.LoadFromStream(Stream: TStream);
begin
  FWaveData.SetSize(Stream.Size);
  Stream.ReadBuffer(FWaveData.Memory^, Stream.Size);
  Changed(Self);
end;

{ method to save WaveAudio data to a stream - used
  by the library and by the SaveToFile method }
procedure TWave.SaveToStream(Stream: TStream);
begin
  Stream.WriteBuffer(FWaveData.Memory^, FWaveData.Size);
end;

{ method to indicate data has changed - triggers
  the OnChange method }
procedure TWave.Changed(Sender: TObject);
begin
  FModified := True;
  if Assigned(FOnChange) then FOnChange(Self);
end;

{ method to allow "fake" data to be read and
  written by the library }
procedure TWave.DefineProperties(Filer: TFiler);
begin
  Filer.DefineBinaryProperty('Data', ReadData, WriteData, not Empty);
end;

{ method to set modified flag }
procedure TWave.SetModified(Value: Boolean);
begin
  if Value then
    Changed(Self) else
    FModified := False;
end;

{ method to read data from *.wav file - calls
  LoadFromStream }
procedure TWave.LoadFromFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmOpenRead);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

{ method to write data to *.wav file - calls
  SaveToStream }
procedure TWave.SaveToFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

{ method for library to read data from stream -
  calls LoadFromStream }
procedure TWave.ReadData(Stream: TStream);
begin
  LoadFromStream(Stream);
end;

{ method for library to write data to stream -
  calls SaveToStream }
procedure TWave.WriteData(Stream: TStream);
begin
  SaveToStream(Stream);
end;

end.
